module  c10lp_gsrd (
        //Clock and Reset
        input  wire        c10_clk50m    ,
        input  wire        hbus_clk_50m  ,  
        input  wire        c10_clk_adj   ,  
        input  wire        enet_clk_125m ,
        input  wire        c10_resetn    ,

        //LED PB DIPSW
        output wire [3:0]  user_led ,
        input  wire [3:0]  user_pb  ,
        input  wire [2:0]  user_dip ,

        //Ethernet Port 
        output wire        enet_mdc        ,
        inout  wire        enet_mdio       ,
        
        input  wire        enet_int        ,     
        output wire        enet_resetn     ,	    
        input  wire        enet_rx_clk     ,     
        output wire        enet_tx_clk     ,    
        input  wire [3:0]  enet_rx_d       ,       
        output wire [3:0]  enet_tx_d       ,       
        output wire        enet_tx_en      ,      
        input  wire        enet_rx_dv      ,      
//        input  wire        enet_led_link100,

        //PMOD PORT
        inout  wire [7:0]  pmod_d   ,

        //Side Bus
        input  wire         usb_reset_n        ,
        input  wire         c10_usb_clk        ,
        input  wire         usb_wr_n           ,
        input  wire         usb_rd_n           ,
        input  wire         usb_oe_n           ,
        output wire         usb_full           ,
        output wire         usb_empty          ,
        inout  wire   [7:0] usb_data           ,
        inout  wire   [1:0] usb_addr           ,
        inout  tri1         usb_scl            ,
        inout  tri1         usb_sda            ,
        
        
        //User IO & Clock
        inout  wire [35:0]  gpio            ,
        
        //ARDUINO IO
        inout  wire [13:0]  arduino_io      ,
        output wire         arduino_rstn    ,
        inout  wire         arduino_sda     ,
        inout  wire         arduino_scl     ,        
        inout  wire         arduino_adc_sda ,
        inout  wire         arduino_adc_scl ,

        //Cyclone 10 to MAX 10 IO
        inout  wire [3:0]  c10_m10_io   ,       
        
        //HyperRAM IO
        output wire         hbus_rstn   ,
        output wire         hbus_clk0p  ,
        output wire         hbus_clk0n  ,
        output wire         hbus_cs2n   , //HyperRAM chip select
        inout  wire         hbus_rwds   ,
        inout  wire [7:0]   hbus_dq     ,
        output wire         hbus_cs1n   , //For HyperFlash
        input  wire         hbus_rston  , //For HyperFlash
        input  wire         hbus_intn     //For HyperFlash

        //QSPI
//        output wire    qspi_dclk   ,
//        output wire    qspi_sce    ,
//        output wire    qspi_sdo    ,
//        input  wire    qspi_data0  

        );

wire i2c_serial_sda_in ;
wire i2c_serial_scl_in ;
wire i2c_serial_sda_oe ;
wire i2c_serial_scl_oe ;
assign i2c_serial_scl_in = arduino_adc_scl;
assign arduino_adc_scl = i2c_serial_scl_oe ? 1'b0 : 1'bz;

assign i2c_serial_sda_in = arduino_adc_sda;
assign arduino_adc_sda = i2c_serial_sda_oe ? 1'b0 : 1'bz;

pll pll_inst0 (
	.areset (!c10_resetn   ) ,
	.inclk0 (enet_clk_125m ) ,
	.c0     (enet_tx_25    ) ,
	.locked (pll_locked    ) );


q_sys               q_sys_inst (
                    .hbus_clk_clk                                                  (c10_clk50m                ),     
                    .hbus_reset_reset_n                                            (c10_resetn                ),
                    .clock_bridge_0_in_clk_clk                                     (enet_tx_25                ), //               clock_bridge_0_in_clk.clk
//                    .ext_epcq_flash_external_dclk                                    (qspi_dclk                 ),
//                    .ext_epcq_flash_external_sce                                     (qspi_sce                  ),
//                    .ext_epcq_flash_external_sdo                                     (qspi_sdo                  ),
//                    .ext_epcq_flash_external_data0                                   (qspi_data0                ),
		                .hyperbus_controller_top_HB_RSTn                                 (hbus_rstn                 ) ,          //             hyperbus_controller_top.HB_RSTn
		                .hyperbus_controller_top_HB_CLK0                                 (hbus_clk0p                ) ,          //                                    .HB_CLK0
		                .hyperbus_controller_top_HB_CLK0n                                (hbus_clk0n                ) ,          //                                    .HB_CLK0n
		                .hyperbus_controller_top_HB_CLK1                                 (                          ) ,          //                                    .HB_CLK1
		                .hyperbus_controller_top_HB_CLK1n                                (                          ) ,          //                                    .HB_CLK1n
		                .hyperbus_controller_top_HB_CS0n                                 (                          ) ,          //                                    .HB_CS0n
		                .hyperbus_controller_top_HB_CS1n                                 (hbus_cs2n                 ) ,          //                                    .HB_CS1n
		                .hyperbus_controller_top_HB_WPn                                  (                          ) ,          //                                    .HB_WPn
		                .hyperbus_controller_top_HB_RWDS                                 (hbus_rwds                 ) ,          //                                    .HB_RWDS
		                .hyperbus_controller_top_HB_dq                                   (hbus_dq                   ) ,          //                                    .HB_dq
		                .hyperbus_controller_top_HB_RSTOn                                (hbus_rston                ) ,          //                                    .HB_RSTOn
		                .hyperbus_controller_top_HB_INTn                                 (hbus_intn                 ) ,          //                                    .HB_INTn
		                .i2c_serial_sda_in                                               (i2c_serial_sda_in         ) ,
		                .i2c_serial_scl_in                                               (i2c_serial_scl_in         ) ,
		                .i2c_serial_sda_oe                                               (i2c_serial_sda_oe         ) ,
		                .i2c_serial_scl_oe                                               (i2c_serial_scl_oe         ) ,
		                .user_dipsw_external_connection_export                           (user_dip                  ) ,
		                .user_pb_external_connection_export                              (user_pb                   ) ,
                    .led_pio_external_connection_export                              (user_led[3:0]             )            //         led_pio_external_connection.export
                    );




endmodule
